from h5py import File, Group
from .metadata import Metadata
from .git import Git
from .interferometer import Interferometer


class Root(Metadata):
    """
    Metadata associated to the Root section
    """

    def __init__(
        self,
        name: str,
        maxtem: int,
        git: Git,
        interferometer: Interferometer,
    ):
        """
        Parameters
        ----------
        name : str
            Unique name to indentify this simulation data
        maxtem : int
            Number of High Order Modes modelised
        git : Git
            Information about the git repo that generated this data
        interferometer : Interferometer
            Information about the simulated interferometer for this data
        """
        self.name = name
        self.maxtem = maxtem
        self.git = git
        self.interferometer = interferometer

    @staticmethod
    def from_hdf5(store: File | Group) -> "Root":
        for attribute in ["name", "maxtem"]:
            if attribute not in store.attrs.keys():
                raise ValueError(
                    "no {} for simulation data".format(attribute)
                )
        name = store.attrs["name"]
        maxtem = store.attrs["maxtem"]

        message = "simulation {} should be a {}, not {}"
        if not isinstance(name, str):
            raise ValueError(
                message.format("name", "string", type(name))
            )
        if not isinstance(maxtem, int):
            raise ValueError(
                message.format("maxtem", "int", type(maxtem))
            )

        git_group = store.get("git")
        interferometer_group = store.get("interferometer")
        message = "{} information should be stored in a group, not {}"
        if not isinstance(git_group, Group):
            raise ValueError(message.format("git", type(git_group)))
        if not isinstance(interferometer_group, Group):
            raise ValueError(
                message.format(
                    "interferometer", type(interferometer_group)
                )
            )
        git = Git.from_hdf5(git_group)
        interferometer = Interferometer.from_hdf5(interferometer_group)
        return Root(
            name=name,
            maxtem=maxtem,
            git=git,
            interferometer=interferometer,
        )

    def to_hdf5(self, store: File | Group) -> Group | File:
        store.attrs["name"] = self.name
        store.attrs["maxtem"] = self.maxtem
        _ = self.git.to_hdf5(store.create_group("git"))
        _ = self.interferometer.to_hdf5(
            store.create_group("interferometer")
        )
        return store
